#ifndef _TSK_LIBTSK_H
#define _TSK_LIBTSK_H

// include the header files for each subdir.
// The _config or _incs.h files are included by base. 
#include "tsk/base/tsk_base.h"
#include "tsk/img/tsk_img.h"
#include "tsk/vs/tsk_vs.h"
#include "tsk/fs/tsk_fs.h"
#include "tsk/hashdb/tsk_hashdb.h"

#endif
